use warnings;
use strict;

package POSIX::Util;
use base 'Exporter';

use POSIX::1003::Pathconf qw/_PC_REC_INCR_XFER_SIZE/;
use POSIX::1003::FdIO     qw/writefd readfd BUFSIZ SSIZE_MAX/;

my @fdio      = qw/readfd_all writefd_all/;
my @functions = (@fdio);

# need to extract the POSIX::1003 exporter
our @EXPORT    = @functions;

our %EXPORT_TAGS =
 ( functions => \@functions
 , fdio      => \@fdio
 );

=chapter NAME

POSIX::Util - Extends

=chapter SYNOPSIS

  use POSIX::Util;

  use POSIX::1003::FdIO qw(openfd O_RDONLY);
  my $fd   = openfd($fn, O_RDONLY);
  defined $fd or die $!;          # $fd is valid!
  my $data = readfd_all $fd, undef, 1;
  defined $data or die $!;

  writefd_all $out, $bytes, 1
      or die "write to $outfile failed: $!\n";
  

=chapter DESCRIPTION
This module collects some functions which are very often needed when working
on the low level of the POSIX interface.  The M<POSIX::1003> module is are
pure as possible, following the standard... this Util module adds some
convenience (at least has that intention).

=chapter FUNCTIONS

=section POSIX::1003::FdIO additions

=function writefd_all FD, BYTES, [DO_CLOSE]
Be sure that BYTES have the utf-8 flag off! We are working with bytes
here, not strings.  Returns undef if something went wrong (error in C<$!>),
otherwise the length of BYTES.

The FD will get closed when DO_CLOSE is provided and true.

=example
  my $out = creatfd $outfile, 0600;
  writefd_all $out, $bytes, 1
      or die "write to $outfile failed: $!\n";
=cut

sub writefd_all($$;$)
{   my ($to, $data, $do_close) = @_;
    my $size = length $data;

    while(my $l = length $data)
    {   my $written = writefd $to, $data, $l;
        defined $written or return;

        last if $l eq $written;    # normal case
        substr($data, 0, $written) = '';
    }

    $do_close && !defined closefd($to) ? undef : $size;
}

=function readfd_all FD, [SIZE, [DO_CLOSE]]
Read all remaining bytes from the FD.  At most SIZE bytes are read,
which defaults to SSIZE_MAX.

The maximum SIZE would be SSIZE_MAX, but POSIX.xs pre-allocs a buffer
with that size, so 2^64 is too large. We will read in convenient

  my $in = openfd $filename, O_RDONLY;
  defined $in or die "cannot open $filename to read: $!\n";

  my $d  = readfd_all $in, undef, 1;
  my $d  = readfd_all $in;
  defined $d or die "cannot read from $filename: $!\n";

=cut

sub readfd_all($;$$)
{   my ($in, $size, $do_close) = @_;
    defined $size or $size = SSIZE_MAX;
    my ($data, $buf) = ('', '');

    my $block = _PC_REC_INCR_XFER_SIZE($in) || BUFSIZ || 4096;

    # we should probably align the $in-fd onto a block size in the
    # first read for optimal performance under all cirmstances...

    my $bytes;
    while($bytes = readfd $in, $buf, ($block < $size ? $block : $size))
    {   last if $bytes==0;   # readfd will return "0 but true"
        $data .= $buf;
        $size -= $bytes;
    }
    defined $bytes or return;

    $do_close && !defined closefd($in) ? undef : $data;
}

1;
